__name__ = "picogo"
__package__ = "picogo"
__version__ = '102'
__author__ = "SkySLAM Co., Ltd."

print( "picogo __init__.py executed." ) 

from .Robot import (
    Robot  
)

print( "Done. picogo __init__.py executed." ) 