if __name__== '__main__' :
    
    print( 'Hello PicoGo!' )
    print( 'Running Bluetooth program ... ' )
    
    from picogo import BlueTooth
    
    BlueTooth.main()
    
    print( 'Good bye!' )
    
pass